# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os
import sys
import pytest
from nipype import config
from unittest.mock import MagicMock

try:
    import xvfbwrapper

    has_Xvfb = True
except ImportError:
    has_Xvfb = False

# Define mocks for xvfbwrapper. Do not forget the spec to ensure that
# hasattr() checks return False with missing attributes.
xvfbpatch = MagicMock(spec=["Xvfb"])
xvfbpatch.Xvfb.return_value = MagicMock(
    spec=["new_display", "start", "stop"], new_display=2010
)

# Mock the legacy xvfbwrapper.Xvfb class (changed display attribute name)
xvfbpatch_old = MagicMock(spec=["Xvfb"])
xvfbpatch_old.Xvfb.return_value = MagicMock(
    spec=["vdisplay_num", "start", "stop"], vdisplay_num=2010
)


@pytest.mark.parametrize("dispvar", [":12", "localhost:12", "localhost:12.1"])
def test_display_parse(monkeypatch, dispvar):
    """Check that when $DISPLAY is defined, the display is correctly parsed"""
    config._display = None
    config._config.remove_option("execution", "display_variable")
    monkeypatch.setenv("DISPLAY", dispvar)
    assert config.get_display() == ":12"
    # Test that it was correctly cached
    assert config.get_display() == ":12"


@pytest.mark.parametrize("dispnum", range(5))
def test_display_config(monkeypatch, dispnum):
    """Check that the display_variable option is used ($DISPLAY not set)"""
    config._display = None
    dispstr = ":%d" % dispnum
    config.set("execution", "display_variable", dispstr)
    monkeypatch.delitem(os.environ, "DISPLAY", raising=False)
    assert config.get_display() == config.get("execution", "display_variable")
    # Test that it was correctly cached
    assert config.get_display() == config.get("execution", "display_variable")


@pytest.mark.parametrize("dispnum", range(5))
def test_display_system(monkeypatch, dispnum):
    """Check that when only a $DISPLAY is defined, it is used"""
    config._display = None
    config._config.remove_option("execution", "display_variable")
    dispstr = ":%d" % dispnum
    monkeypatch.setenv("DISPLAY", dispstr)
    assert config.get_display() == dispstr
    # Test that it was correctly cached
    assert config.get_display() == dispstr


def test_display_config_and_system(monkeypatch):
    """Check that when only both config and $DISPLAY are defined, the config
    takes precedence"""
    config._display = None
    dispstr = ":10"
    config.set("execution", "display_variable", dispstr)
    monkeypatch.setenv("DISPLAY", ":0")
    assert config.get_display() == dispstr
    # Test that it was correctly cached
    assert config.get_display() == dispstr


def test_display_noconfig_nosystem_patched(monkeypatch):
    """Check that when no $DISPLAY nor option are specified, a virtual Xvfb is
    used"""
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.delitem(os.environ, "DISPLAY", raising=False)
    monkeypatch.setitem(sys.modules, "xvfbwrapper", xvfbpatch)
    monkeypatch.setattr(sys, "platform", value="linux")
    assert config.get_display() == ":2010"
    # Test that it was correctly cached
    assert config.get_display() == ":2010"

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, "platform", value="darwin")
    with pytest.raises(RuntimeError):
        config.get_display()


def test_display_empty_patched(monkeypatch):
    """
    Check that when $DISPLAY is empty string and no option is specified,
    a virtual Xvfb is used
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.setenv("DISPLAY", "")
    monkeypatch.setitem(sys.modules, "xvfbwrapper", xvfbpatch)
    monkeypatch.setattr(sys, "platform", value="linux")
    assert config.get_display() == ":2010"
    # Test that it was correctly cached
    assert config.get_display() == ":2010"

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, "platform", value="darwin")
    with pytest.raises(RuntimeError):
        config.get_display()


def test_display_noconfig_nosystem_patched_oldxvfbwrapper(monkeypatch):
    """
    Check that when no $DISPLAY nor option are specified,
    a virtual Xvfb is used (with a legacy version of xvfbwrapper).
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.delitem(os.environ, "DISPLAY", raising=False)
    monkeypatch.setitem(sys.modules, "xvfbwrapper", xvfbpatch_old)
    monkeypatch.setattr(sys, "platform", value="linux")
    assert config.get_display() == ":2010"
    # Test that it was correctly cached
    assert config.get_display() == ":2010"

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, "platform", value="darwin")
    with pytest.raises(RuntimeError):
        config.get_display()


def test_display_empty_patched_oldxvfbwrapper(monkeypatch):
    """
    Check that when $DISPLAY is empty string and no option is specified,
    a virtual Xvfb is used (with a legacy version of xvfbwrapper).
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.setenv("DISPLAY", "")
    monkeypatch.setitem(sys.modules, "xvfbwrapper", xvfbpatch_old)
    monkeypatch.setattr(sys, "platform", value="linux")
    assert config.get_display() == ":2010"
    # Test that it was correctly cached
    assert config.get_display() == ":2010"

    # Check that raises in Mac
    config._display = None
    monkeypatch.setattr(sys, "platform", value="darwin")
    with pytest.raises(RuntimeError):
        config.get_display()


def test_display_noconfig_nosystem_notinstalled(monkeypatch):
    """
    Check that an exception is raised if xvfbwrapper is not installed
    but necessary (no config and $DISPLAY unset)
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.delenv("DISPLAY", raising=False)
    monkeypatch.setitem(sys.modules, "xvfbwrapper", None)
    with pytest.raises(RuntimeError):
        config.get_display()


def test_display_empty_notinstalled(monkeypatch):
    """
    Check that an exception is raised if xvfbwrapper is not installed
    but necessary (no config and $DISPLAY empty)
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.setenv("DISPLAY", "")
    monkeypatch.setitem(sys.modules, "xvfbwrapper", None)
    with pytest.raises(RuntimeError):
        config.get_display()


@pytest.mark.skipif(not has_Xvfb, reason="xvfbwrapper not installed")
@pytest.mark.skipif("darwin" in sys.platform, reason="macosx requires root for Xvfb")
def test_display_noconfig_nosystem_installed(monkeypatch):
    """
    Check that actually uses xvfbwrapper when installed (not mocked)
    and necessary (no config and $DISPLAY unset)
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.delenv("DISPLAY", raising=False)
    newdisp = config.get_display()
    assert int(newdisp.split(":")[-1]) > 1000
    # Test that it was correctly cached
    assert config.get_display() == newdisp


@pytest.mark.skipif(not has_Xvfb, reason="xvfbwrapper not installed")
@pytest.mark.skipif("darwin" in sys.platform, reason="macosx requires root for Xvfb")
def test_display_empty_installed(monkeypatch):
    """
    Check that actually uses xvfbwrapper when installed (not mocked)
    and necessary (no config and $DISPLAY empty)
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.setenv("DISPLAY", "")
    newdisp = config.get_display()
    assert int(newdisp.split(":")[-1]) > 1000
    # Test that it was correctly cached
    assert config.get_display() == newdisp


def test_display_empty_macosx(monkeypatch):
    """
    Check that an exception is raised if xvfbwrapper is necessary
    (no config and $DISPLAY unset) but platform is OSX. See
    https://github.com/nipy/nipype/issues/1400
    """
    config._display = None
    if config.has_option("execution", "display_variable"):
        config._config.remove_option("execution", "display_variable")
    monkeypatch.delenv("DISPLAY", "")

    monkeypatch.setattr(sys, "platform", "darwin")
    with pytest.raises(RuntimeError):
        config.get_display()


def test_cwd_cached(tmpdir):
    """Check that changing dirs does not change nipype's cwd"""
    oldcwd = config.cwd
    tmpdir.chdir()
    assert config.cwd == oldcwd


def test_debug_mode():
    from ... import logging

    sofc_config = config.get("execution", "stop_on_first_crash")
    ruo_config = config.get("execution", "remove_unnecessary_outputs")
    ki_config = config.get("execution", "keep_inputs")
    wf_config = config.get("logging", "workflow_level")
    if_config = config.get("logging", "interface_level")
    ut_config = config.get("logging", "utils_level")

    wf_level = logging.getLogger("nipype.workflow").level
    if_level = logging.getLogger("nipype.interface").level
    ut_level = logging.getLogger("nipype.utils").level

    config.enable_debug_mode()

    # Check config is updated and logging levels, too
    assert config.get("execution", "stop_on_first_crash") == "true"
    assert config.get("execution", "remove_unnecessary_outputs") == "false"
    assert config.get("execution", "keep_inputs") == "true"
    assert config.get("logging", "workflow_level") == "DEBUG"
    assert config.get("logging", "interface_level") == "DEBUG"
    assert config.get("logging", "utils_level") == "DEBUG"

    assert logging.getLogger("nipype.workflow").level == 10
    assert logging.getLogger("nipype.interface").level == 10
    assert logging.getLogger("nipype.utils").level == 10

    # Restore config and levels
    config.set("execution", "stop_on_first_crash", sofc_config)
    config.set("execution", "remove_unnecessary_outputs", ruo_config)
    config.set("execution", "keep_inputs", ki_config)
    config.set("logging", "workflow_level", wf_config)
    config.set("logging", "interface_level", if_config)
    config.set("logging", "utils_level", ut_config)
    logging.update_logging(config)

    assert config.get("execution", "stop_on_first_crash") == sofc_config
    assert config.get("execution", "remove_unnecessary_outputs") == ruo_config
    assert config.get("execution", "keep_inputs") == ki_config
    assert config.get("logging", "workflow_level") == wf_config
    assert config.get("logging", "interface_level") == if_config
    assert config.get("logging", "utils_level") == ut_config

    assert logging.getLogger("nipype.workflow").level == wf_level
    assert logging.getLogger("nipype.interface").level == if_level
    assert logging.getLogger("nipype.utils").level == ut_level



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
import os
import warnings

import pytest

from ....utils.filemanip import split_filename
from ... import base as nib
from ...base import traits, Undefined
from ....interfaces import fsl
from ...utility.wrappers import Function
from ....pipeline import Node
from ..specs import get_filecopy_info


@pytest.fixture(scope="module")
def setup_file(request, tmpdir_factory):
    tmp_dir = tmpdir_factory.mktemp("files")
    tmp_infile = tmp_dir.join("foo.txt")
    with tmp_infile.open("w") as fp:
        fp.writelines(["123456789"])

    tmp_dir.chdir()

    return tmp_infile.strpath


def test_TraitedSpec():
    assert nib.TraitedSpec().get_hashval()
    assert nib.TraitedSpec().__repr__() == "\n\n"

    class spec(nib.TraitedSpec):
        foo = nib.traits.Int
        goo = nib.traits.Float(usedefault=True)

    assert spec().foo == Undefined
    assert spec().goo == 0.0
    specfunc = lambda x: spec(hoo=x)
    with pytest.raises(nib.traits.TraitError):
        specfunc(1)
    infields = spec(foo=1)
    hashval = (
        [("foo", 1), ("goo", "0.0000000000")],
        "e89433b8c9141aa0fda2f8f4d662c047",
    )
    assert infields.get_hashval() == hashval
    assert infields.__repr__() == "\nfoo = 1\ngoo = 0.0\n"


def test_TraitedSpec_tab_completion():
    bet_nd = Node(fsl.BET(), name="bet")
    bet_interface = fsl.BET()
    bet_inputs = bet_nd.inputs.class_editable_traits()
    bet_outputs = bet_nd.outputs.class_editable_traits()

    # Check __all__ for bet node and interface inputs
    assert set(bet_nd.inputs.__all__) == set(bet_inputs)
    assert set(bet_interface.inputs.__all__) == set(bet_inputs)

    # Check __all__ for bet node outputs
    assert set(bet_nd.outputs.__all__) == set(bet_outputs)


@pytest.mark.skip
def test_TraitedSpec_dynamic():
    from pickle import dumps, loads

    a = nib.BaseTraitedSpec()
    a.add_trait("foo", nib.traits.Int)
    a.foo = 1
    assign_a = lambda: setattr(a, "foo", "a")
    with pytest.raises(Exception):
        assign_a
    pkld_a = dumps(a)
    unpkld_a = loads(pkld_a)
    assign_a_again = lambda: setattr(unpkld_a, "foo", "a")
    with pytest.raises(Exception):
        assign_a_again


def test_DynamicTraitedSpec_tab_completion():
    def extract_func(list_out):
        return list_out[0]

    # Define interface
    func_interface = Function(
        input_names=["list_out"],
        output_names=["out_file", "another_file"],
        function=extract_func,
    )
    # Define node
    list_extract = Node(
        Function(
            input_names=["list_out"], output_names=["out_file"], function=extract_func
        ),
        name="list_extract",
    )

    # Check __all__ for interface inputs
    expected_input = set(list_extract.inputs.editable_traits())
    assert set(func_interface.inputs.__all__) == expected_input

    # Check __all__ for node inputs
    assert set(list_extract.inputs.__all__) == expected_input

    # Check __all__ for node outputs
    expected_output = set(list_extract.outputs.editable_traits())
    assert set(list_extract.outputs.__all__) == expected_output

    # Add trait and retest
    list_extract._interface._output_names.append("added_out_trait")
    expected_output.add("added_out_trait")
    assert set(list_extract.outputs.__all__) == expected_output


def test_TraitedSpec_logic():
    class spec3(nib.TraitedSpec):
        _xor_inputs = ("foo", "bar")

        foo = nib.traits.Int(xor=_xor_inputs, desc="foo or bar, not both")
        bar = nib.traits.Int(xor=_xor_inputs, desc="bar or foo, not both")
        kung = nib.traits.Float(requires=("foo",), position=0, desc="kung foo")

    class out3(nib.TraitedSpec):
        output = nib.traits.Int

    class MyInterface(nib.BaseInterface):
        input_spec = spec3
        output_spec = out3

    myif = MyInterface()
    # NOTE_dj, FAIL: I don't get a TypeError, only a UserWarning
    # with pytest.raises(TypeError):
    #     setattr(myif.inputs, 'kung', 10.0)
    myif.inputs.foo = 1
    assert myif.inputs.foo == 1
    set_bar = lambda: setattr(myif.inputs, "bar", 1)
    with pytest.raises(IOError):
        set_bar()
    assert myif.inputs.foo == 1
    myif.inputs.kung = 2
    assert myif.inputs.kung == 2.0


def test_deprecation():
    with warnings.catch_warnings(record=True) as w:
        warnings.filterwarnings("always", "", UserWarning)

        class DeprecationSpec1(nib.TraitedSpec):
            foo = nib.traits.Int(deprecated="0.1")

        spec_instance = DeprecationSpec1()
        set_foo = lambda: setattr(spec_instance, "foo", 1)
        with pytest.raises(nib.TraitError):
            set_foo()
        assert len(w) == 0, "no warnings, just errors"

    with warnings.catch_warnings(record=True) as w:
        warnings.filterwarnings("always", "", UserWarning)

        class DeprecationSpec2(nib.TraitedSpec):
            foo = nib.traits.Int(deprecated="100", new_name="bar")

        spec_instance = DeprecationSpec2()
        set_foo = lambda: setattr(spec_instance, "foo", 1)
        with pytest.raises(nib.TraitError):
            set_foo()
        assert len(w) == 0, "no warnings, just errors"

    with warnings.catch_warnings(record=True) as w:
        warnings.filterwarnings("always", "", UserWarning)

        class DeprecationSpec3(nib.TraitedSpec):
            foo = nib.traits.Int(deprecated="1000", new_name="bar")
            bar = nib.traits.Int()

        spec_instance = DeprecationSpec3()
        not_raised = True
        try:
            spec_instance.foo = 1
        except nib.TraitError:
            not_raised = False
        assert not_raised
        assert len(w) == 1, f"deprecated warning 1 {[str(w1) for w1 in w]}"

    with warnings.catch_warnings(record=True) as w:
        warnings.filterwarnings("always", "", UserWarning)

        class DeprecationSpec3(nib.TraitedSpec):
            foo = nib.traits.Int(deprecated="1000", new_name="bar")
            bar = nib.traits.Int()

        spec_instance = DeprecationSpec3()
        not_raised = True
        try:
            spec_instance.foo = 1
        except nib.TraitError:
            not_raised = False
        assert not_raised
        assert spec_instance.foo == Undefined
        assert spec_instance.bar == 1
        assert len(w) == 1, f"deprecated warning 2 {[str(w1) for w1 in w]}"


def test_namesource(setup_file):
    tmp_infile = setup_file
    tmpd, nme, ext = split_filename(tmp_infile)

    class spec2(nib.CommandLineInputSpec):
        moo = nib.File(name_source=["doo"], hash_files=False, argstr="%s", position=2)
        doo = nib.File(exists=True, argstr="%s", position=1)
        goo = traits.Int(argstr="%d", position=4)
        poo = nib.File(name_source=["goo"], hash_files=False, argstr="%s", position=3)

    class TestName(nib.CommandLine):
        _cmd = "mycommand"
        input_spec = spec2

    testobj = TestName()
    testobj.inputs.doo = tmp_infile
    testobj.inputs.goo = 99
    assert "%s_generated" % nme in testobj.cmdline
    assert "%d_generated" % testobj.inputs.goo in testobj.cmdline
    testobj.inputs.moo = "my_%s_template"
    assert "my_%s_template" % nme in testobj.cmdline


def test_chained_namesource(setup_file):
    tmp_infile = setup_file
    tmpd, nme, ext = split_filename(tmp_infile)

    class spec2(nib.CommandLineInputSpec):
        doo = nib.File(exists=True, argstr="%s", position=1)
        moo = nib.File(
            name_source=["doo"],
            hash_files=False,
            argstr="%s",
            position=2,
            name_template="%s_mootpl",
        )
        poo = nib.File(name_source=["moo"], hash_files=False, argstr="%s", position=3)

    class TestName(nib.CommandLine):
        _cmd = "mycommand"
        input_spec = spec2

    testobj = TestName()
    testobj.inputs.doo = tmp_infile
    res = testobj.cmdline
    assert "%s" % tmp_infile in res
    assert "%s_mootpl " % nme in res
    assert "%s_mootpl_generated" % nme in res


def test_cycle_namesource1(setup_file):
    tmp_infile = setup_file
    tmpd, nme, ext = split_filename(tmp_infile)

    class spec3(nib.CommandLineInputSpec):
        moo = nib.File(
            name_source=["doo"],
            hash_files=False,
            argstr="%s",
            position=1,
            name_template="%s_mootpl",
        )
        poo = nib.File(name_source=["moo"], hash_files=False, argstr="%s", position=2)
        doo = nib.File(name_source=["poo"], hash_files=False, argstr="%s", position=3)

    class TestCycle(nib.CommandLine):
        _cmd = "mycommand"
        input_spec = spec3

    # Check that an exception is raised
    to0 = TestCycle()
    not_raised = True
    try:
        to0.cmdline
    except nib.NipypeInterfaceError:
        not_raised = False
    assert not not_raised


def test_cycle_namesource2(setup_file):
    tmp_infile = setup_file
    tmpd, nme, ext = split_filename(tmp_infile)

    class spec3(nib.CommandLineInputSpec):
        moo = nib.File(
            name_source=["doo"],
            hash_files=False,
            argstr="%s",
            position=1,
            name_template="%s_mootpl",
        )
        poo = nib.File(name_source=["moo"], hash_files=False, argstr="%s", position=2)
        doo = nib.File(name_source=["poo"], hash_files=False, argstr="%s", position=3)

    class TestCycle(nib.CommandLine):
        _cmd = "mycommand"
        input_spec = spec3

    # Check that loop can be broken by setting one of the inputs
    to1 = TestCycle()
    to1.inputs.poo = tmp_infile

    not_raised = True
    try:
        res = to1.cmdline
    except nib.NipypeInterfaceError:
        not_raised = False
    print(res)

    assert not_raised
    assert "%s" % tmp_infile in res
    assert "%s_generated" % nme in res
    assert "%s_generated_mootpl" % nme in res


def test_namesource_constraints(setup_file):
    tmp_infile = setup_file
    tmpd, nme, ext = split_filename(tmp_infile)

    class constrained_spec(nib.CommandLineInputSpec):
        in_file = nib.File(argstr="%s", position=1)
        threshold = traits.Float(argstr="%g", xor=["mask_file"], position=2)
        mask_file = nib.File(
            argstr="%s",
            name_source=["in_file"],
            name_template="%s_mask",
            keep_extension=True,
            xor=["threshold"],
            position=2,
        )
        out_file1 = nib.File(
            argstr="%s",
            name_source=["in_file"],
            name_template="%s_out1",
            keep_extension=True,
            position=3,
        )
        out_file2 = nib.File(
            argstr="%s",
            name_source=["in_file"],
            name_template="%s_out2",
            keep_extension=True,
            requires=["threshold"],
            position=4,
        )

    class TestConstrained(nib.CommandLine):
        _cmd = "mycommand"
        input_spec = constrained_spec

    tc = TestConstrained()

    # name_source undefined, so template traits remain undefined
    assert tc.cmdline == "mycommand"

    # mask_file and out_file1 enabled by name_source definition
    tc.inputs.in_file = os.path.basename(tmp_infile)
    assert tc.cmdline == "mycommand foo.txt foo_mask.txt foo_out1.txt"

    # mask_file disabled by threshold, out_file2 enabled by threshold
    tc.inputs.threshold = 10.0
    assert tc.cmdline == "mycommand foo.txt 10 foo_out1.txt foo_out2.txt"


def test_TraitedSpec_withFile(setup_file):
    tmp_infile = setup_file
    tmpd, nme = os.path.split(tmp_infile)
    assert os.path.exists(tmp_infile)

    class spec2(nib.TraitedSpec):
        moo = nib.File(exists=True)
        doo = nib.traits.List(nib.File(exists=True))

    infields = spec2(moo=tmp_infile, doo=[tmp_infile])
    hashval = infields.get_hashval(hash_method="content")
    assert hashval[1] == "a00e9ee24f5bfa9545a515b7a759886b"


def test_TraitedSpec_withNoFileHashing(setup_file):
    tmp_infile = setup_file
    tmpd, nme = os.path.split(tmp_infile)
    assert os.path.exists(tmp_infile)

    class spec2(nib.TraitedSpec):
        moo = nib.File(exists=True, hash_files=False)
        doo = nib.traits.List(nib.File(exists=True))

    infields = spec2(moo=nme, doo=[tmp_infile])
    hashval = infields.get_hashval(hash_method="content")
    assert hashval[1] == "8da4669ff5d72f670a46ea3e7a203215"

    class spec3(nib.TraitedSpec):
        moo = nib.File(exists=True, name_source="doo")
        doo = nib.traits.List(nib.File(exists=True))

    infields = spec3(moo=nme, doo=[tmp_infile])
    hashval1 = infields.get_hashval(hash_method="content")

    class spec4(nib.TraitedSpec):
        moo = nib.File(exists=True)
        doo = nib.traits.List(nib.File(exists=True))

    infields = spec4(moo=nme, doo=[tmp_infile])
    hashval2 = infields.get_hashval(hash_method="content")
    assert hashval1[1] != hashval2[1]


def test_ImageFile():
    x = nib.BaseInterface().inputs

    # setup traits
    x.add_trait("nifti", nib.ImageFile(types=["nifti1", "dicom"]))
    x.add_trait("anytype", nib.ImageFile())
    with pytest.raises(ValueError):
        x.add_trait("newtype", nib.ImageFile(types=["nifti10"]))
    x.add_trait("nocompress", nib.ImageFile(types=["mgh"], allow_compressed=False))

    with pytest.raises(nib.TraitError):
        x.nifti = "test.mgz"
    x.nifti = "test.nii"
    x.anytype = "test.xml"
    with pytest.raises(nib.TraitError):
        x.nocompress = "test.mgz"
    x.nocompress = "test.mgh"


def test_filecopy_info():
    class InputSpec(nib.TraitedSpec):
        foo = nib.traits.Int(desc="a random int")
        goo = nib.traits.Int(desc="a random int", mandatory=True)
        moo = nib.traits.Int(desc="a random int", mandatory=False)
        hoo = nib.traits.Int(desc="a random int", usedefault=True)
        zoo = nib.File(desc="a file", copyfile=False)
        woo = nib.File(desc="a file", copyfile=True)

    class DerivedInterface(nib.BaseInterface):
        input_spec = InputSpec
        resource_monitor = False

        def normalize_filenames(self):
            """A mock normalize_filenames for freesurfer interfaces that have one"""
            self.inputs.zoo = "normalized_filename.ext"

    assert get_filecopy_info(nib.BaseInterface) == []

    # Test on interface class, not instantiated
    info = get_filecopy_info(DerivedInterface)
    assert info[0]["key"] == "woo"
    assert info[0]["copy"]
    assert info[1]["key"] == "zoo"
    assert not info[1]["copy"]
    info = None

    # Test with instantiated interface
    derived = DerivedInterface()
    # First check that zoo is not defined
    assert derived.inputs.zoo == Undefined
    # After the first call to get_filecopy_info zoo is defined
    info = get_filecopy_info(derived)
    # Ensure that normalize_filenames was called
    assert derived.inputs.zoo == "normalized_filename.ext"
    # Check the results are consistent
    assert info[0]["key"] == "woo"
    assert info[0]["copy"]
    assert info[1]["key"] == "zoo"
    assert not info[1]["copy"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:

import os

from nipype.testing import example_data

import numpy as np


def test_overlap(tmpdir):
    from nipype.algorithms.metrics import Overlap

    def check_close(val1, val2):
        import numpy.testing as npt

        return npt.assert_almost_equal(val1, val2, decimal=3)

    in1 = example_data("segmentation0.nii.gz")
    in2 = example_data("segmentation1.nii.gz")

    tmpdir.chdir()
    overlap = Overlap()
    overlap.inputs.volume1 = in1
    overlap.inputs.volume2 = in1
    res = overlap.run()
    check_close(res.outputs.jaccard, 1.0)

    overlap = Overlap()
    overlap.inputs.volume1 = in1
    overlap.inputs.volume2 = in2
    res = overlap.run()
    check_close(res.outputs.jaccard, 0.99705)

    overlap = Overlap()
    overlap.inputs.volume1 = in1
    overlap.inputs.volume2 = in2
    overlap.inputs.vol_units = "mm"
    res = overlap.run()
    check_close(res.outputs.jaccard, 0.99705)
    check_close(res.outputs.roi_voldiff, np.array([0.0063086, -0.0025506, 0.0]))



# -*- coding: utf-8 -*-
# emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
# vi: set ft=python sts=4 ts=4 sw=4 et:
"""
Pytest fixtures used in tests.
"""
import os
import pytest
import numpy as np
import nibabel as nb

from nipype.utils.filemanip import ensure_list
from nipype.interfaces.fsl import Info
from nipype.interfaces.fsl.base import FSLCommand


def analyze_pair_image_files(outdir, filelist, shape):
    for f in ensure_list(filelist):
        hdr = nb.Nifti1Header()
        hdr.set_data_shape(shape)
        img = np.random.random(shape)
        analyze = nb.AnalyzeImage(img, np.eye(4), hdr)
        analyze.to_filename(os.path.join(outdir, f))


def nifti_image_files(outdir, filelist, shape):
    for f in ensure_list(filelist):
        img = np.random.random(shape)
        nb.Nifti1Image(img, np.eye(4), None).to_filename(os.path.join(outdir, f))


@pytest.fixture()
def create_files_in_directory(request, tmpdir):
    cwd = tmpdir.chdir()
    filelist = ["a.nii", "b.nii"]
    nifti_image_files(tmpdir.strpath, filelist, shape=(3, 3, 3, 4))

    def change_directory():
        cwd.chdir()

    request.addfinalizer(change_directory)
    return (filelist, tmpdir.strpath)


@pytest.fixture()
def create_analyze_pair_file_in_directory(request, tmpdir):
    cwd = tmpdir.chdir()
    filelist = ["a.hdr"]
    analyze_pair_image_files(tmpdir.strpath, filelist, shape=(3, 3, 3, 4))

    def change_directory():
        cwd.chdir()

    request.addfinalizer(change_directory)
    return (filelist, tmpdir.strpath)


@pytest.fixture()
def create_files_in_directory_plus_dummy_file(request, tmpdir):
    cwd = tmpdir.chdir()
    filelist = ["a.nii", "b.nii"]
    nifti_image_files(tmpdir.strpath, filelist, shape=(3, 3, 3, 4))

    tmpdir.join("reg.dat").write("dummy file")
    filelist.append("reg.dat")

    def change_directory():
        cwd.chdir()

    request.addfinalizer(change_directory)
    return (filelist, tmpdir.strpath)


@pytest.fixture()
def create_surf_file_in_directory(request, tmpdir):
    cwd = tmpdir.chdir()
    surf = "lh.a.nii"
    nifti_image_files(tmpdir.strpath, filelist=surf, shape=(1, 100, 1))

    def change_directory():
        cwd.chdir()

    request.addfinalizer(change_directory)
    return (surf, tmpdir.strpath)


def set_output_type(fsl_output_type):
    prev_output_type = os.environ.get("FSLOUTPUTTYPE", None)

    if fsl_output_type is not None:
        os.environ["FSLOUTPUTTYPE"] = fsl_output_type
    elif "FSLOUTPUTTYPE" in os.environ:
        del os.environ["FSLOUTPUTTYPE"]

    FSLCommand.set_default_output_type(Info.output_type())
    return prev_output_type


@pytest.fixture(params=[None] + sorted(Info.ftypes))
def create_files_in_directory_plus_output_type(request, tmpdir):
    func_prev_type = set_output_type(request.param)
    origdir = tmpdir.chdir()
    filelist = ["a.nii", "b.nii"]
    nifti_image_files(tmpdir.strpath, filelist, shape=(3, 3, 3, 4))

    out_ext = Info.output_type_to_ext(Info.output_type())

    def fin():
        set_output_type(func_prev_type)
        origdir.chdir()

    request.addfinalizer(fin)
    return (filelist, tmpdir.strpath, out_ext)



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..registration import AffineTask


def test_AffineTask_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        fixed_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=0,
        ),
        ftol=dict(
            argstr="%g",
            mandatory=True,
            position=4,
            usedefault=True,
        ),
        initialize_xfm=dict(
            argstr="%s",
            copyfile=True,
            extensions=None,
            position=5,
        ),
        moving_file=dict(
            argstr="%s",
            copyfile=False,
            extensions=None,
            mandatory=True,
            position=1,
        ),
        sampling_xyz=dict(
            argstr="%g %g %g",
            mandatory=True,
            position=3,
            usedefault=True,
        ),
        similarity_metric=dict(
            argstr="%s",
            mandatory=True,
            position=2,
            usedefault=True,
        ),
    )
    inputs = AffineTask.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_AffineTask_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
        out_file_xfm=dict(
            extensions=None,
        ),
    )
    outputs = AffineTask.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..rapidart import ArtifactDetect


def test_ArtifactDetect_inputs():
    input_map = dict(
        bound_by_brainmask=dict(
            usedefault=True,
        ),
        global_threshold=dict(
            usedefault=True,
        ),
        intersect_mask=dict(
            usedefault=True,
        ),
        mask_file=dict(
            extensions=None,
        ),
        mask_threshold=dict(),
        mask_type=dict(
            mandatory=True,
        ),
        norm_threshold=dict(
            mandatory=True,
            xor=["rotation_threshold", "translation_threshold"],
        ),
        parameter_source=dict(
            mandatory=True,
        ),
        plot_type=dict(
            usedefault=True,
        ),
        realigned_files=dict(
            mandatory=True,
        ),
        realignment_parameters=dict(
            mandatory=True,
        ),
        rotation_threshold=dict(
            mandatory=True,
            xor=["norm_threshold"],
        ),
        save_plot=dict(
            usedefault=True,
        ),
        translation_threshold=dict(
            mandatory=True,
            xor=["norm_threshold"],
        ),
        use_differences=dict(
            maxlen=2,
            minlen=2,
            usedefault=True,
        ),
        use_norm=dict(
            requires=["norm_threshold"],
            usedefault=True,
        ),
        zintensity_threshold=dict(
            mandatory=True,
        ),
    )
    inputs = ArtifactDetect.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ArtifactDetect_outputs():
    output_map = dict(
        displacement_files=dict(),
        intensity_files=dict(),
        mask_files=dict(),
        norm_files=dict(),
        outlier_files=dict(),
        plot_files=dict(),
        statistic_files=dict(),
    )
    outputs = ArtifactDetect.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import ReHo


def test_ReHo_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        chi_sq=dict(
            argstr="-chi_sq",
        ),
        ellipsoid=dict(
            argstr="-neigh_X %s -neigh_Y %s -neigh_Z %s",
            xor=["sphere", "neighborhood"],
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-inset %s",
            extensions=None,
            mandatory=True,
            position=1,
        ),
        label_set=dict(
            argstr="-in_rois %s",
            extensions=None,
        ),
        mask_file=dict(
            argstr="-mask %s",
            extensions=None,
        ),
        neighborhood=dict(
            argstr="-nneigh %s",
            xor=["sphere", "ellipsoid"],
        ),
        out_file=dict(
            argstr="-prefix %s",
            extensions=None,
            keep_extension=True,
            name_source="in_file",
            name_template="%s_reho",
            position=0,
        ),
        overwrite=dict(
            argstr="-overwrite",
        ),
        sphere=dict(
            argstr="-neigh_RAD %s",
            xor=["neighborhood", "ellipsoid"],
        ),
    )
    inputs = ReHo.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_ReHo_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
        out_vals=dict(
            extensions=None,
        ),
    )
    outputs = ReHo.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



import os
from pathlib import Path

from nipype.interfaces.base import File, InputMultiPath, TraitedSpec, traits, isdefined
from nipype.interfaces.cat12.base import NestedCell, Cell
from nipype.interfaces.spm import SPMCommand
from nipype.interfaces.spm.base import SPMCommandInputSpec
from nipype.utils.filemanip import split_filename


class ExtractAdditionalSurfaceParametersInputSpec(SPMCommandInputSpec):
    left_central_surfaces = InputMultiPath(
        File(exists=True),
        field="data_surf",
        desc="Left and central surfaces files",
        mandatory=True,
        copyfile=False,
    )
    surface_files = InputMultiPath(
        File(exists=True), desc="All surface files", mandatory=False, copyfile=False
    )

    gyrification = traits.Bool(
        True,
        field="GI",
        usedefault=True,
        desc="Extract gyrification index (GI) based on absolute mean curvature. The"
        " method is described in Luders et al. Neuroimage, 29:1224-1230, 2006",
    )
    gmv = traits.Bool(True, field="gmv", usedefault=True, desc="Extract volume")
    area = traits.Bool(True, field="area", usedefault=True, desc="Extract area surface")
    depth = traits.Bool(
        False,
        field="SD",
        usedefault=True,
        desc="Extract sulcus depth based on euclidian distance between the central "
        "surface anf its convex hull.",
    )
    fractal_dimension = traits.Bool(
        False,
        field="FD",
        usedefault=True,
        desc="Extract cortical complexity (fractal dimension) which is "
        "described in Yotter ar al. Neuroimage, 56(3): 961-973, 2011",
    )


class ExtractAdditionalSurfaceParametersOutputSpec(TraitedSpec):
    lh_extracted_files = traits.List(
        File(exists=True), desc="Files of left Hemisphere extracted measures"
    )
    rh_extracted_files = traits.List(
        File(exists=True), desc="Files of right Hemisphere extracted measures"
    )

    lh_gyrification = traits.List(
        File(exists=True), desc="Gyrification of left Hemisphere"
    )
    rh_gyrification = traits.List(
        File(exists=True), desc="Gyrification of right Hemisphere"
    )

    lh_gmv = traits.List(
        File(exists=True), desc="Grey matter volume of left Hemisphere"
    )
    rh_gmv = traits.List(
        File(exists=True), desc="Grey matter volume of right Hemisphere"
    )

    lh_area = traits.List(File(exists=True), desc="Area of left Hemisphere")
    rh_area = traits.List(File(exists=True), desc="Area of right Hemisphere")

    lh_depth = traits.List(File(exists=True), desc="Depth of left Hemisphere")
    rh_depth = traits.List(File(exists=True), desc="Depth of right Hemisphere")

    lh_fractaldimension = traits.List(
        File(exists=True), desc="Fractal Dimension of left Hemisphere"
    )
    rh_fractaldimension = traits.List(
        File(exists=True), desc="Fractal Dimension of right Hemisphere"
    )


class ExtractAdditionalSurfaceParameters(SPMCommand):
    """
    Additional surface parameters can be extracted that can be used for statistical analysis, such as:

    * Central surfaces
    * Surface area
    * Surface GM volume
    * Gyrification Index
    * Sulcus depth
    * Toro's gyrification index
    * Shaer's local gyrification index
    * Laplacian gyrification indeces
    * Addicional surfaces
    * Measure normalization
    * Lazy processing

    http://www.neuro.uni-jena.de/cat12/CAT12-Manual.pdf#page=53

    Examples
    --------
    >>> # Set the left surface files, both will be processed
    >>> lh_path_central = 'lh.central.structural.gii'
    >>> # Put here all surface files generated by CAT12 Segment, this is only required if the this approach is putted in
    >>> surf_files = ['lh.sphere.reg.structural.gii', 'rh.sphere.reg.structural.gii', 'lh.sphere.structural.gii', 'rh.sphere.structural.gii', 'rh.central.structural.gii', 'lh.pbt.structural', 'rh.pbt.structural']
    >>> extract_additional_measures = ExtractAdditionalSurfaceParameters(left_central_surfaces=lh_path_central, surface_files=surf_files)
    >>> extract_additional_measures.run() # doctest: +SKIP

    """

    input_spec = ExtractAdditionalSurfaceParametersInputSpec
    output_spec = ExtractAdditionalSurfaceParametersOutputSpec

    def __init__(self, **inputs):
        _local_version = SPMCommand().version
        if _local_version and "12." in _local_version:
            self._jobtype = "tools"
            self._jobname = "cat.stools.surfextract"

        super().__init__(**inputs)

    def _list_outputs(self):
        outputs = self._outputs().get()

        names_outputs = [
            (self.inputs.gyrification, "gyrification"),
            (self.inputs.gmv, "gmv"),
            (self.inputs.area, "area"),
            (self.inputs.depth, "depth"),
            (self.inputs.fractal_dimension, "fractaldimension"),
        ]

        for filename in self.inputs.left_central_surfaces:
            pth, base, ext = split_filename(filename)
            # The first part of the filename is rh.central or lh.central
            original_filename = base.split(".", 2)[-1]
            for extracted_parameter, parameter_name in names_outputs:
                if extracted_parameter:
                    for hemisphere in ["rh", "lh"]:
                        all_files_hemisphere = hemisphere + "_extracted_files"
                        name_hemisphere = hemisphere + "_" + parameter_name
                        if not isdefined(outputs[name_hemisphere]):
                            outputs[name_hemisphere] = []
                        if not isdefined(outputs[all_files_hemisphere]):
                            outputs[all_files_hemisphere] = []
                        generated_filename = ".".join(
                            [hemisphere, parameter_name, original_filename]
                        )
                        outputs[name_hemisphere].append(
                            os.path.join(pth, generated_filename)
                        )

                        # Add all hemisphere files into one list, this is important because only the left hemisphere
                        # files are used as input in the Surface ROI Tools, fpr instance.
                        outputs[all_files_hemisphere].append(
                            os.path.join(pth, generated_filename)
                        )

        return outputs

    def _format_arg(self, opt, spec, val):
        if opt == "left_central_surfaces":
            return Cell2Str(val)
        return super(ExtractAdditionalSurfaceParameters, self)._format_arg(
            opt, spec, val
        )


class ExtractROIBasedSurfaceMeasuresInputSpec(SPMCommandInputSpec):
    # Only these files are given as input, yet the right hemisphere (rh) files should also be on the processing
    # directory.

    surface_files = InputMultiPath(
        File(exists=True),
        desc="Surface data files. This variable should be a list " "with all",
        mandatory=False,
        copyfile=False,
    )
    lh_roi_atlas = InputMultiPath(
        File(exists=True),
        field="rdata",
        desc="(Left) ROI Atlas. These are the ROI's ",
        mandatory=True,
        copyfile=False,
    )

    rh_roi_atlas = InputMultiPath(
        File(exists=True),
        desc="(Right) ROI Atlas. These are the ROI's ",
        mandatory=False,
        copyfile=False,
    )

    lh_surface_measure = InputMultiPath(
        File(exists=True),
        field="cdata",
        desc="(Left) Surface data files. ",
        mandatory=True,
        copyfile=False,
    )
    rh_surface_measure = InputMultiPath(
        File(exists=True),
        desc="(Right) Surface data files.",
        mandatory=False,
        copyfile=False,
    )


class ExtractROIBasedSurfaceMeasuresOutputSpec(TraitedSpec):
    label_files = traits.List(
        File(exists=True), desc="Files with the measures extracted for ROIs."
    )


class ExtractROIBasedSurfaceMeasures(SPMCommand):
    """
    Extract ROI-based surface values
    While ROI-based values for VBM (volume) data are automatically saved in the ``label`` folder as XML file it is
    necessary to additionally extract these values for surface data (except for thickness which is automatically
    extracted during segmentation). This has to be done after preprocessing the data and creating cortical surfaces.

    You can extract ROI-based values for cortical thickness but also for any other surface parameter that was extracted
    using the Extract Additional Surface Parameters such as volume, area, depth, gyrification and fractal dimension.


     http://www.neuro.uni-jena.de/cat12/CAT12-Manual.pdf#page=53

     Examples
     --------
    >>> # Template surface files
    >>> lh_atlas = 'lh.aparc_a2009s.freesurfer.annot'
    >>> rh_atlas = 'rh.aparc_a2009s.freesurfer.annot'
    >>> surf_files = ['lh.sphere.reg.structural.gii', 'rh.sphere.reg.structural.gii', 'lh.sphere.structural.gii', 'rh.sphere.structural.gii', 'lh.central.structural.gii', 'rh.central.structural.gii', 'lh.pbt.structural', 'rh.pbt.structural']
    >>> lh_measure = 'lh.area.structural'
    >>> extract_additional_measures = ExtractROIBasedSurfaceMeasures(surface_files=surf_files, lh_surface_measure=lh_measure, lh_roi_atlas=lh_atlas, rh_roi_atlas=rh_atlas)
    >>> extract_additional_measures.run() # doctest: +SKIP


    """

    input_spec = ExtractROIBasedSurfaceMeasuresInputSpec
    output_spec = ExtractROIBasedSurfaceMeasuresOutputSpec

    def __init__(self, **inputs):
        _local_version = SPMCommand().version
        if _local_version and "12." in _local_version:
            self._jobtype = "tools"
            self._jobname = "cat.stools.surf2roi"

        SPMCommand.__init__(self, **inputs)

    def _format_arg(self, opt, spec, val):
        if opt == "lh_surface_measure":
            return NestedCell(val)
        elif opt == "lh_roi_atlas":
            return Cell2Str(val)

        return super(ExtractROIBasedSurfaceMeasures, self)._format_arg(opt, spec, val)

    def _list_outputs(self):
        outputs = self._outputs().get()

        pth, base, ext = split_filename(self.inputs.lh_surface_measure[0])

        outputs["label_files"] = [
            str(label) for label in Path(pth).glob("label/*") if label.is_file()
        ]
        return outputs


class Cell2Str(Cell):
    def __str__(self):
        """Convert input to appropriate format for cat12"""
        return "{%s}" % self.to_string()



# -*- coding: utf-8 -*-
# -*- coding: utf8 -*-
"""Autogenerated file - DO NOT EDIT
If you spot a bug, please report it on the mailing list and/or change the generator."""

import os

from ...base import (
    CommandLine,
    CommandLineInputSpec,
    SEMLikeCommandLine,
    TraitedSpec,
    File,
    Directory,
    traits,
    isdefined,
    InputMultiPath,
    OutputMultiPath,
)


class GenerateSummedGradientImageInputSpec(CommandLineInputSpec):
    inputVolume1 = File(
        desc="input volume 1, usally t1 image", exists=True, argstr="--inputVolume1 %s"
    )
    inputVolume2 = File(
        desc="input volume 2, usally t2 image", exists=True, argstr="--inputVolume2 %s"
    )
    outputFileName = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="(required) output file name",
        argstr="--outputFileName %s",
    )
    MaximumGradient = traits.Bool(
        desc="If set this flag, it will compute maximum gradient between two input volumes instead of sum of it.",
        argstr="--MaximumGradient ",
    )
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d",
    )


class GenerateSummedGradientImageOutputSpec(TraitedSpec):
    outputFileName = File(desc="(required) output file name", exists=True)


class GenerateSummedGradientImage(SEMLikeCommandLine):
    """title: GenerateSummedGradient

    category: Filtering.FeatureDetection

    description: Automatic FeatureImages using neural networks

    version: 1.0

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Greg Harris, Eun Young Kim
    """

    input_spec = GenerateSummedGradientImageInputSpec
    output_spec = GenerateSummedGradientImageOutputSpec
    _cmd = " GenerateSummedGradientImage "
    _outputs_filenames = {"outputFileName": "outputFileName"}
    _redirect_x = False


class CannySegmentationLevelSetImageFilterInputSpec(CommandLineInputSpec):
    inputVolume = File(exists=True, argstr="--inputVolume %s")
    initialModel = File(exists=True, argstr="--initialModel %s")
    outputVolume = traits.Either(
        traits.Bool, File(), hash_files=False, argstr="--outputVolume %s"
    )
    outputSpeedVolume = traits.Either(
        traits.Bool, File(), hash_files=False, argstr="--outputSpeedVolume %s"
    )
    cannyThreshold = traits.Float(
        desc="Canny Threshold Value", argstr="--cannyThreshold %f"
    )
    cannyVariance = traits.Float(desc="Canny variance", argstr="--cannyVariance %f")
    advectionWeight = traits.Float(
        desc="Controls the smoothness of the resulting mask, small number are more smooth, large numbers allow more sharp corners.  ",
        argstr="--advectionWeight %f",
    )
    initialModelIsovalue = traits.Float(
        desc="The identification of the input model iso-surface.  (for a binary image with 0s and 1s use 0.5) (for a binary image with 0s and 255's use 127.5).",
        argstr="--initialModelIsovalue %f",
    )
    maxIterations = traits.Int(desc="The", argstr="--maxIterations %d")


class CannySegmentationLevelSetImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(exists=True)
    outputSpeedVolume = File(exists=True)


class CannySegmentationLevelSetImageFilter(SEMLikeCommandLine):
    """title: Canny Level Set Image Filter

    category: Filtering.FeatureDetection

    description: The CannySegmentationLevelSet is commonly used to refine a manually generated manual mask.

    version: 0.3.0

    license: CC

    contributor: Regina Kim

    acknowledgements: This command module was derived from Insight/Examples/Segmentation/CannySegmentationLevelSetImageFilter.cxx (copyright) Insight Software Consortium.  See http://wiki.na-mic.org/Wiki/index.php/Slicer3:Execution_Model_Documentation for more detailed descriptions.
    """

    input_spec = CannySegmentationLevelSetImageFilterInputSpec
    output_spec = CannySegmentationLevelSetImageFilterOutputSpec
    _cmd = " CannySegmentationLevelSetImageFilter "
    _outputs_filenames = {
        "outputVolume": "outputVolume.nii",
        "outputSpeedVolume": "outputSpeedVolume.nii",
    }
    _redirect_x = False


class DilateImageInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    inputMaskVolume = File(
        desc="Required: input brain mask image",
        exists=True,
        argstr="--inputMaskVolume %s",
    )
    inputRadius = traits.Int(
        desc="Required: input neighborhood radius", argstr="--inputRadius %d"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class DilateImageOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class DilateImage(SEMLikeCommandLine):
    """title: Dilate Image

    category: Filtering.FeatureDetection

    description: Uses mathematical morphology to dilate the input images.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = DilateImageInputSpec
    output_spec = DilateImageOutputSpec
    _cmd = " DilateImage "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class TextureFromNoiseImageFilterInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    inputRadius = traits.Int(
        desc="Required: input neighborhood radius", argstr="--inputRadius %d"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class TextureFromNoiseImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class TextureFromNoiseImageFilter(SEMLikeCommandLine):
    """title: TextureFromNoiseImageFilter

    category: Filtering.FeatureDetection

    description: Calculate the local noise in an image.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Eunyoung Regina Kim
    """

    input_spec = TextureFromNoiseImageFilterInputSpec
    output_spec = TextureFromNoiseImageFilterOutputSpec
    _cmd = " TextureFromNoiseImageFilter "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class FlippedDifferenceInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    inputMaskVolume = File(
        desc="Required: input brain mask image",
        exists=True,
        argstr="--inputMaskVolume %s",
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class FlippedDifferenceOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class FlippedDifference(SEMLikeCommandLine):
    """title: Flip Image

    category: Filtering.FeatureDetection

    description: Difference between an image and the axially flipped version of that image.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = FlippedDifferenceInputSpec
    output_spec = FlippedDifferenceOutputSpec
    _cmd = " FlippedDifference "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class ErodeImageInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    inputMaskVolume = File(
        desc="Required: input brain mask image",
        exists=True,
        argstr="--inputMaskVolume %s",
    )
    inputRadius = traits.Int(
        desc="Required: input neighborhood radius", argstr="--inputRadius %d"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class ErodeImageOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class ErodeImage(SEMLikeCommandLine):
    """title: Erode Image

    category: Filtering.FeatureDetection

    description: Uses mathematical morphology to erode the input images.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = ErodeImageInputSpec
    output_spec = ErodeImageOutputSpec
    _cmd = " ErodeImage "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class GenerateBrainClippedImageInputSpec(CommandLineInputSpec):
    inputImg = File(
        desc="input volume 1, usally t1 image", exists=True, argstr="--inputImg %s"
    )
    inputMsk = File(
        desc="input volume 2, usally t2 image", exists=True, argstr="--inputMsk %s"
    )
    outputFileName = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="(required) output file name",
        argstr="--outputFileName %s",
    )
    numberOfThreads = traits.Int(
        desc="Explicitly specify the maximum number of threads to use.",
        argstr="--numberOfThreads %d",
    )


class GenerateBrainClippedImageOutputSpec(TraitedSpec):
    outputFileName = File(desc="(required) output file name", exists=True)


class GenerateBrainClippedImage(SEMLikeCommandLine):
    """title: GenerateBrainClippedImage

    category: Filtering.FeatureDetection

    description: Automatic FeatureImages using neural networks

    version: 1.0

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Eun Young Kim
    """

    input_spec = GenerateBrainClippedImageInputSpec
    output_spec = GenerateBrainClippedImageOutputSpec
    _cmd = " GenerateBrainClippedImage "
    _outputs_filenames = {"outputFileName": "outputFileName"}
    _redirect_x = False


class NeighborhoodMedianInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    inputMaskVolume = File(
        desc="Required: input brain mask image",
        exists=True,
        argstr="--inputMaskVolume %s",
    )
    inputRadius = traits.Int(
        desc="Required: input neighborhood radius", argstr="--inputRadius %d"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class NeighborhoodMedianOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class NeighborhoodMedian(SEMLikeCommandLine):
    """title: Neighborhood Median

    category: Filtering.FeatureDetection

    description: Calculates the median, for the given neighborhood size, at each voxel of the input image.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = NeighborhoodMedianInputSpec
    output_spec = NeighborhoodMedianOutputSpec
    _cmd = " NeighborhoodMedian "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class GenerateTestImageInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="input volume 1, usally t1 image", exists=True, argstr="--inputVolume %s"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="(required) output file name",
        argstr="--outputVolume %s",
    )
    lowerBoundOfOutputVolume = traits.Float(argstr="--lowerBoundOfOutputVolume %f")
    upperBoundOfOutputVolume = traits.Float(argstr="--upperBoundOfOutputVolume %f")
    outputVolumeSize = traits.Float(
        desc="output Volume Size", argstr="--outputVolumeSize %f"
    )


class GenerateTestImageOutputSpec(TraitedSpec):
    outputVolume = File(desc="(required) output file name", exists=True)


class GenerateTestImage(SEMLikeCommandLine):
    """title: DownSampleImage

    category: Filtering.FeatureDetection

    description: Down sample image for testing

    version: 1.0

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: Eun Young Kim
    """

    input_spec = GenerateTestImageInputSpec
    output_spec = GenerateTestImageOutputSpec
    _cmd = " GenerateTestImage "
    _outputs_filenames = {"outputVolume": "outputVolume"}
    _redirect_x = False


class NeighborhoodMeanInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    inputMaskVolume = File(
        desc="Required: input brain mask image",
        exists=True,
        argstr="--inputMaskVolume %s",
    )
    inputRadius = traits.Int(
        desc="Required: input neighborhood radius", argstr="--inputRadius %d"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class NeighborhoodMeanOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class NeighborhoodMean(SEMLikeCommandLine):
    """title: Neighborhood Mean

    category: Filtering.FeatureDetection

    description: Calculates the mean, for the given neighborhood size, at each voxel of the T1, T2, and FLAIR.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = NeighborhoodMeanInputSpec
    output_spec = NeighborhoodMeanOutputSpec
    _cmd = " NeighborhoodMean "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class HammerAttributeCreatorInputSpec(CommandLineInputSpec):
    Scale = traits.Int(desc="Determine Scale of Ball", argstr="--Scale %d")
    Strength = traits.Float(desc="Determine Strength of Edges", argstr="--Strength %f")
    inputGMVolume = File(
        desc="Required: input grey matter posterior image",
        exists=True,
        argstr="--inputGMVolume %s",
    )
    inputWMVolume = File(
        desc="Required: input white matter posterior image",
        exists=True,
        argstr="--inputWMVolume %s",
    )
    inputCSFVolume = File(
        desc="Required: input CSF posterior image",
        exists=True,
        argstr="--inputCSFVolume %s",
    )
    outputVolumeBase = traits.Str(
        desc="Required: output image base name to be appended for each feature vector.",
        argstr="--outputVolumeBase %s",
    )


class HammerAttributeCreatorOutputSpec(TraitedSpec):
    pass


class HammerAttributeCreator(SEMLikeCommandLine):
    """title: HAMMER Feature Vectors

    category: Filtering.FeatureDetection

    description: Create the feature vectors used by HAMMER.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This was extracted from the Hammer Registration source code, and wrapped up by Hans J. Johnson.
    """

    input_spec = HammerAttributeCreatorInputSpec
    output_spec = HammerAttributeCreatorOutputSpec
    _cmd = " HammerAttributeCreator "
    _outputs_filenames = {}
    _redirect_x = False


class TextureMeasureFilterInputSpec(CommandLineInputSpec):
    inputVolume = File(exists=True, argstr="--inputVolume %s")
    inputMaskVolume = File(exists=True, argstr="--inputMaskVolume %s")
    distance = traits.Int(argstr="--distance %d")
    insideROIValue = traits.Float(argstr="--insideROIValue %f")
    outputFilename = traits.Either(
        traits.Bool, File(), hash_files=False, argstr="--outputFilename %s"
    )


class TextureMeasureFilterOutputSpec(TraitedSpec):
    outputFilename = File(exists=True)


class TextureMeasureFilter(SEMLikeCommandLine):
    """title: Canny Level Set Image Filter

    category: Filtering.FeatureDetection

    description: The CannySegmentationLevelSet is commonly used to refine a manually generated manual mask.

    version: 0.3.0

    license: CC

    contributor: Regina Kim

    acknowledgements: This command module was derived from Insight/Examples/Segmentation/CannySegmentationLevelSetImageFilter.cxx (copyright) Insight Software Consortium.  See http://wiki.na-mic.org/Wiki/index.php/Slicer3:Execution_Model_Documentation for more detailed descriptions.
    """

    input_spec = TextureMeasureFilterInputSpec
    output_spec = TextureMeasureFilterOutputSpec
    _cmd = " TextureMeasureFilter "
    _outputs_filenames = {"outputFilename": "outputFilename"}
    _redirect_x = False


class DilateMaskInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    inputBinaryVolume = File(
        desc="Required: input brain mask image",
        exists=True,
        argstr="--inputBinaryVolume %s",
    )
    sizeStructuralElement = traits.Int(
        desc="size of structural element. sizeStructuralElement=1 means that 3x3x3 structuring element for 3D",
        argstr="--sizeStructuralElement %d",
    )
    lowerThreshold = traits.Float(
        desc="Required: lowerThreshold value", argstr="--lowerThreshold %f"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class DilateMaskOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class DilateMask(SEMLikeCommandLine):
    """title: Dilate Image

    category: Filtering.FeatureDetection

    description: Uses mathematical morphology to dilate the input images.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = DilateMaskInputSpec
    output_spec = DilateMaskOutputSpec
    _cmd = " DilateMask "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class DumpBinaryTrainingVectorsInputSpec(CommandLineInputSpec):
    inputHeaderFilename = File(
        desc="Required: input header file name",
        exists=True,
        argstr="--inputHeaderFilename %s",
    )
    inputVectorFilename = File(
        desc="Required: input vector filename",
        exists=True,
        argstr="--inputVectorFilename %s",
    )


class DumpBinaryTrainingVectorsOutputSpec(TraitedSpec):
    pass


class DumpBinaryTrainingVectors(SEMLikeCommandLine):
    """title: Erode Image

    category: Filtering.FeatureDetection

    description: Uses mathematical morphology to erode the input images.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = DumpBinaryTrainingVectorsInputSpec
    output_spec = DumpBinaryTrainingVectorsOutputSpec
    _cmd = " DumpBinaryTrainingVectors "
    _outputs_filenames = {}
    _redirect_x = False


class DistanceMapsInputSpec(CommandLineInputSpec):
    inputLabelVolume = File(
        desc="Required: input tissue label image",
        exists=True,
        argstr="--inputLabelVolume %s",
    )
    inputMaskVolume = File(
        desc="Required: input brain mask image",
        exists=True,
        argstr="--inputMaskVolume %s",
    )
    inputTissueLabel = traits.Int(
        desc="Required: input integer value of tissue type used to calculate distance",
        argstr="--inputTissueLabel %d",
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class DistanceMapsOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class DistanceMaps(SEMLikeCommandLine):
    """title: Mauerer Distance

    category: Filtering.FeatureDetection

    description: Get the distance from a voxel to the nearest voxel of a given tissue type.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = DistanceMapsInputSpec
    output_spec = DistanceMapsOutputSpec
    _cmd = " DistanceMaps "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class STAPLEAnalysisInputSpec(CommandLineInputSpec):
    inputDimension = traits.Int(
        desc="Required: input image Dimension 2 or 3", argstr="--inputDimension %d"
    )
    inputLabelVolume = InputMultiPath(
        File(exists=True),
        desc="Required: input label volume",
        argstr="--inputLabelVolume %s...",
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class STAPLEAnalysisOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class STAPLEAnalysis(SEMLikeCommandLine):
    """title: Dilate Image

    category: Filtering.FeatureDetection

    description: Uses mathematical morphology to dilate the input images.

    version: 0.1.0.$Revision: 1 $(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was developed by Mark Scully and Jeremy Bockholt.
    """

    input_spec = STAPLEAnalysisInputSpec
    output_spec = STAPLEAnalysisOutputSpec
    _cmd = " STAPLEAnalysis "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class GradientAnisotropicDiffusionImageFilterInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input image", exists=True, argstr="--inputVolume %s"
    )
    numberOfIterations = traits.Int(
        desc="Optional value for number of Iterations", argstr="--numberOfIterations %d"
    )
    timeStep = traits.Float(
        desc="Time step for diffusion process", argstr="--timeStep %f"
    )
    conductance = traits.Float(
        desc="Conductance for diffusion process", argstr="--conductance %f"
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class GradientAnisotropicDiffusionImageFilterOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class GradientAnisotropicDiffusionImageFilter(SEMLikeCommandLine):
    """title: GradientAnisopropicDiffusionFilter

    category: Filtering.FeatureDetection

    description: Image Smoothing using Gradient Anisotropic Diffuesion Filer

    contributor: This tool was developed by Eun Young Kim by modifying ITK Example
    """

    input_spec = GradientAnisotropicDiffusionImageFilterInputSpec
    output_spec = GradientAnisotropicDiffusionImageFilterOutputSpec
    _cmd = " GradientAnisotropicDiffusionImageFilter "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False


class CannyEdgeInputSpec(CommandLineInputSpec):
    inputVolume = File(
        desc="Required: input tissue label image",
        exists=True,
        argstr="--inputVolume %s",
    )
    variance = traits.Float(
        desc="Variance and Maximum error are used in the Gaussian smoothing of the input image.  See  itkDiscreteGaussianImageFilter for information on these parameters.",
        argstr="--variance %f",
    )
    upperThreshold = traits.Float(
        desc="Threshold is the lowest allowed value in the output image.  Its data type is the same as the data type of the output image. Any values below the Threshold level will be replaced with the OutsideValue parameter value, whose default is zero.  ",
        argstr="--upperThreshold %f",
    )
    lowerThreshold = traits.Float(
        desc="Threshold is the lowest allowed value in the output image.  Its data type is the same as the data type of the output image. Any values below the Threshold level will be replaced with the OutsideValue parameter value, whose default is zero.  ",
        argstr="--lowerThreshold %f",
    )
    outputVolume = traits.Either(
        traits.Bool,
        File(),
        hash_files=False,
        desc="Required: output image",
        argstr="--outputVolume %s",
    )


class CannyEdgeOutputSpec(TraitedSpec):
    outputVolume = File(desc="Required: output image", exists=True)


class CannyEdge(SEMLikeCommandLine):
    """title: Canny Edge Detection

    category: Filtering.FeatureDetection

    description: Get the distance from a voxel to the nearest voxel of a given tissue type.

    version: 0.1.0.(alpha)

    documentation-url: http:://www.na-mic.org/

    license: https://www.nitrc.org/svn/brains/BuildScripts/trunk/License.txt

    contributor: This tool was written by Hans J. Johnson.
    """

    input_spec = CannyEdgeInputSpec
    output_spec = CannyEdgeOutputSpec
    _cmd = " CannyEdge "
    _outputs_filenames = {"outputVolume": "outputVolume.nii"}
    _redirect_x = False



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..connectivity import BuildConnectome


def test_BuildConnectome_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-3,
        ),
        in_parc=dict(
            argstr="%s",
            extensions=None,
            position=-2,
        ),
        in_scalar=dict(
            argstr="-image %s",
            extensions=None,
        ),
        in_weights=dict(
            argstr="-tck_weights_in %s",
            extensions=None,
        ),
        keep_unassigned=dict(
            argstr="-keep_unassigned",
        ),
        metric=dict(
            argstr="-metric %s",
        ),
        nthreads=dict(
            argstr="-nthreads %d",
            nohash=True,
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-1,
            usedefault=True,
        ),
        search_forward=dict(
            argstr="-assignment_forward_search %f",
        ),
        search_radius=dict(
            argstr="-assignment_radial_search %f",
        ),
        search_reverse=dict(
            argstr="-assignment_reverse_search %f",
        ),
        vox_lookup=dict(
            argstr="-assignment_voxel_lookup",
        ),
        zero_diagonal=dict(
            argstr="-zero_diagonal",
        ),
    )
    inputs = BuildConnectome.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_BuildConnectome_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = BuildConnectome.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..tracking import FilterTracks


def test_FilterTracks_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        debug=dict(
            argstr="-debug",
            position=1,
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        exclude_file=dict(
            argstr="-exclude %s",
            extensions=None,
            xor=["exclude_file", "exclude_spec"],
        ),
        exclude_spec=dict(
            argstr="-exclude %s",
            position=2,
            sep=",",
            units="mm",
            xor=["exclude_file", "exclude_spec"],
        ),
        in_file=dict(
            argstr="%s",
            extensions=None,
            mandatory=True,
            position=-2,
        ),
        include_file=dict(
            argstr="-include %s",
            extensions=None,
            xor=["include_file", "include_spec"],
        ),
        include_spec=dict(
            argstr="-include %s",
            position=2,
            sep=",",
            units="mm",
            xor=["include_file", "include_spec"],
        ),
        invert=dict(
            argstr="-invert",
        ),
        minimum_tract_length=dict(
            argstr="-minlength %s",
            units="mm",
        ),
        no_mask_interpolation=dict(
            argstr="-nomaskinterp",
        ),
        out_file=dict(
            argstr="%s",
            extensions=None,
            hash_files=False,
            name_source=["in_file"],
            name_template="%s_filt",
            position=-1,
        ),
        quiet=dict(
            argstr="-quiet",
            position=1,
        ),
    )
    inputs = FilterTracks.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_FilterTracks_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = FilterTracks.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value



# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..utils import SigLoss


def test_SigLoss_inputs():
    input_map = dict(
        args=dict(
            argstr="%s",
        ),
        echo_time=dict(
            argstr="--te=%f",
        ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        in_file=dict(
            argstr="-i %s",
            extensions=None,
            mandatory=True,
        ),
        mask_file=dict(
            argstr="-m %s",
            extensions=None,
        ),
        out_file=dict(
            argstr="-s %s",
            extensions=None,
            genfile=True,
        ),
        output_type=dict(),
        slice_direction=dict(
            argstr="-d %s",
        ),
    )
    inputs = SigLoss.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_SigLoss_outputs():
    output_map = dict(
        out_file=dict(
            extensions=None,
        ),
    )
    outputs = SigLoss.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
